//++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*! \file	Rule.cpp
 *  \brief Implementacia triedy CRule - pravidlo
 *  \author Rado Kostelnik
 *  \date 1.12.2001 1:20:25
 */
//++++++++++++++++++++++++++++++++++++++++++++++++++++++

//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRule::CRule(CRuleEngine* parent) : m_Parent(parent)
{

}

CRule::~CRule()
{
	m_conds.clear();
	m_actions.clear();
}

//------------------------------------------------------
/*! \fn	bool CRule::set_XML( IXMLDOMNodePtr e )
 *  \brief Cita udaje pravidla z XML elementu
 *  \param 
 *  \return
 *  \author Rado Kostelnik
 *  \date 1.12.2001 1:13:05
 */
//------------------------------------------------------
/* Historia zmien:
 */
//------------------------------------------------------
bool CRule::set_XML( IXMLDOMNodePtr e )
{
	_variant_t v;
	IXMLDOMNodeListPtr l;
	IXMLDOMElementPtr el;

	//
	// read attributes
	//

	e->attributes->getNamedItem( "name" )->get_nodeValue( &v );

	m_Name = _bstr_t(v);

	//
	// read conditions
	//

	l = e->selectNodes( L"conds/cond" );

	l->reset();

	el = l->nextNode(); 

	while ( el )
	{
		CCondition *r = new CCondition;
		r->set_XML ( el );
		m_conds.push_back( *r );

		delete r;
		el.Release();
		el = l->nextNode(); 
	}

	if ( el )
		el.Release();
	if ( l )
		l.Release();
	//
	// read actions
	//
	l = e->selectNodes( L"actions/*" );

	l->reset();

	el = l->nextNode(); 

	while ( el )
	{
		CAction* a = new CAction;
		a->set_XML( el );
		m_actions.push_back( *a );

		delete a;
		el.Release();
		el = l->nextNode(); 
	}

	if ( el )
		el.Release();
	if ( l )
		l.Release();

	//e.Release();

	return true;
}

//------------------------------------------------------
/*! \fn	TActionVector* CRule::Evaluate( CQuery& query )
 *  \brief Vyhodnotenie podmienky
 *  \param query Objekt, ktory vracia hodnoty premennych
 *  \return Vracia vector akcii a ich parametrov
 *  \author Rado Kostelnik
 *  \date 1.12.2001 1:13:43
 */
//------------------------------------------------------
/* Historia zmien:
 */
//------------------------------------------------------
TActionVector* CRule::Evaluate( CQuery& query )
{
	TConditionVectorIterator i = m_conds.begin();
	TConditionVectorIterator e = m_conds.end();

#ifdef DEBUG
	cout << m_Name << endl;
#endif

	// 
	// evaluate conditions
	//
	for ( ; i!=e; i++ )
	{
		CCondition& c = *i;
		if ( c.evaluate( query ) == false )
			return NULL;
	}

	TActionVectorIterator ai = m_actions.begin();
	TActionVectorIterator ae = m_actions.end();
	TActionVector* av = new TActionVector;

	//
	// Generate action list
	//
	for ( ; ai != ae; ai++ )
	{
		CAction& action = *ai;
		
		//
		// Get all rules by name
		//
		TRuleVector* rules = m_Parent->GetRulesByName( action.name );
		
		if ( rules->size() == 0  )
		{
			//
			// if there are no rules with this name, 
			// consider the action is elementary and add it to the list
			//
			av->push_back( action );	
		}
		else
		{
			//
			// else evaluate the rules
			//
			TRuleVectorIterator ri = rules->begin();
			TRuleVectorIterator re = rules->end();

			for ( ; ri != re; ri++ )
			{
				CRule& r = *ri;
				TActionVector* avtmp = NULL;
				
				avtmp = r.Evaluate( query );

				if ( avtmp )
				{
					// 
					// if the rule has true executable conditions
					// add its result actions to the list
					// 
					TActionVectorIterator aitmp = avtmp->begin();
					TActionVectorIterator aetmp = avtmp->end();

					for ( ; aitmp != aetmp; aitmp++ )
					{
						CAction& actmp = *aitmp;
						av->push_back( actmp );						
					}

					delete avtmp;
					break;
				}
			}
		}
		delete rules;
	}

	return av;
}
